function [Ih,Ihc,xB,Xny,Rny,P,n,i]=opdatering_final(X,Xny,IX,Iy,Iex,Ih,...
    Ihc,beta,Rny,K,n,xB,P,tau,i,bins,n_in_bin)

% Call:
% 
% [Ih,Ihc,xB,Xny,Rny,P,n,i]=opdatering_final(X,Xny,IX,Iy,Iex,Ih,...
%    Ihc,beta,Rny,K,n,xB,P,tau,i,bins,n_in_bin)
%
% This function update the elements needed for the adaptive simplex
% procedure in rq_simplex_final. 
%
% Input    :
% X        : A matrix with all the data to be simulated, the columns IX are
%            the columns needeed use in the quantile regression, column is 
%            the corresponding respons and column Iex is used for updating
%            in several bins.
% Xny      : The "design matrix" that have to be updated, Xny(1:end,IX) is 
%            the actual design matrix, see "X" for the other columns
% IX       : Xny(1:end,IX) is the design matrix
% Iy       : Xny(1:end,Iy) is the respons 
% Iex      : Xny(1:end,Iex) is the an explanatory variable used for updating
%            in several bins
% Ih       : An index set s.t. Xny(Ih,IX)^(-1)*Xny(Ih,Iy)=beta
% Ihc      : The complement of Ih, i.e. Ihc={1,2,...,n}\Ih where n is the
%            number of rows in Xny.
% beta     : the coefficients in the quantile regression problem
% Rny      : A vector containing previous one step ahead prediction errors
% K        : K=rank(X(1:end,IX))=rank(Xny(Ih,IX))
% n        : The number of observations in the design matrix Xny
% xB       : A vector s.t xB=[beta' abs(r(Ihc))']', where r is the residuals
%           for the quantile regression model, corresponding to Xny
% P        : P=sign(r(Ihc)), with sign(0)=1
% tau      : The required probability in the quantile regressioin problem 
% i        : Internal counter for the rq methods s.t.
%            Xny(end,1:end)=X(n+i-2,1:end)
% bins     : A partion of the sample space of the variable in X(Iex, ), to
%            make sure this is the case bins should be a vector with
%            -Inf=bins(1)<bins(2)<...<bins(end)=Inf
% n_in_bin : the number of elements allowed in the bins (one number)
%
% The output is updated versions of (some of) the inputs.
%
% The algorithm follows the algorithms described in [1].
%
% References:
% [1] J. K. Mller (2006), Modeling of Unceartainty in Wind Energy
%        Forecast. Master Thesis, Informatics and Mathematical Modelling,
%        Technical University of Denmark. Available at
%        http://www.imm.dtu.dk/pubdb/p.php?4428. 
%
% [2] H. B. Nielsen (1999), Algorithms for Linear Optimization, an
%       Introduction. Course note for the DTU course "Optimization and Data
%       fitting 2". Available at http://www.imm.dtu.dk/courses/02611/ 

Xny=[Xny;X(n+i-1,1:end)]; % The newest avaliable observation is added to 
                          % the design matrix
                          
Index=1:length(Xny(1:end-1,Iy)); % An indexset corresponding to the old 
                                 % design matrix is created
                                 
              
j=1;                        % The while loop determines a number j s.t. 
while Xny(end,Iex)>bins(j)  % the new explanatory variable is in the 
    j=j+1;                  % interval (bins(j);bins(j+1)]
end
j=j-1;

 % The index set of the design matrix in this interval is determined and
 % the oldest element in the interval is marked for deletion.
In=Index(Xny(1:end-1,Iex)>bins(j) & Xny(1:end-1,Iex)<=bins(j+1));
Leav=min(In);
  
  % minIL is a marker if minIL=0 then the leaving variable is in Ih, and it
  % cannot be removed before a simplex step have been taken away from this.
[minIL,Inmin]=min(abs(Ih-Leav)); 
 
  % The if statement take one simplex step with an objective function with
  % zero loss on the leaving variable. This simplex step ensure that the
  % row marked for deletion can leave the design matrix.
if minIL==0 & length(In)==n_in_bin
    cB=P<0+P.*tau;
    invXh=Xny(Ih,IX)^-1;
    g=-(P.*(Xny(Ihc,IX)*invXh(1:end,Inmin)))'*cB;
    h=[invXh(1:end,Inmin);-P.*(Xny(Ihc,IX)*invXh(1:end,Inmin))]*sign(g);
    sigma=zeros(1,n-K);
    hm=h(K+1:end);
    xBm=xB(K+1:end);
    xBm(xBm<0)=0;
    sigma(hm>10^-10)=xBm(hm>10^-10)./hm(hm>10^-10);
    sigma(hm<=10^-10)=Inf;
    [alpha,q]=min(sigma);
    
    z=xB-alpha*h;
    Ihm=Ih(Inmin);
    Ih(Inmin)=Ihc(q);
    Ihc(q)=Ihm;
    
    xB=z;
    xB(q+K)=alpha;
    
    P(q)=sign(g)+g==0;
    Ih=sort(Ih);
    [Ihc,IndexIhc]=sort(Ihc);
    P=P(IndexIhc);
    xBm=xB(K+1:end);
    xBm=xBm(IndexIhc);
    xB(K+1:end)=xBm;
    beta=xB(1:K);
end

 % The residual of the one step ahead predition is calculated 
rny=X(n+i-1,Iy)-X(n+i-1,IX)*beta;
Rny=[Rny,rny];

 % This residual is used to update P and xB
if rny<0
    P=[P;-1];
    xB=[xB;-rny];
else
    P=[P;1];
    xB=[xB;rny];
end
Ihc(end+1)=n+1;
    
 % If the bin is filled then the updating of the indexsets is a little
 % tricky since not all of the indexset is updated, if the bin is not 
 % filled then updating is straight forward. See below.
if length(In)==n_in_bin
    i=i+1;
    Stay=ones(length(Ihc),1);
    Stay(Ihc==Leav)=0;
    Stay=Stay==1;
    P=P(Stay);
    Ihc=Ihc(Stay);
    Xny=Xny(sort([Ih,Ihc]),1:end);
    xBm=xB(K+1:end);
    xBm=xBm(Stay);
    xB=xB(1:end-1);
    xB(K+1:end)=xBm;
    Ihc(Ihc>Leav)=Ihc(Ihc>Leav)-1;
    Ih(Ih>Leav)=Ih(Ih>Leav)-1;
else
    n=n+1;
end
