function [xB,Ih,Ihc,P]=rq_initialiser_final(X,r,beta,n)
% Call:
%
% [xB,Ih,Ihc,P]=rq_initialiser_final(X,r,beta,K,n)
%
% rq_initialiser_final uses the a design matrix and the residuals from a
% quantile regression problem:
%
% y=X*beta+r
%
% I.e. to give something meaningfull r must contain at least rank(X) zeros. 
%
% The input to the function is (with n equal to the number of elements in 
% r)
% 
% X    : the design matrix or X(1:n,1:end) is the design matrix
% r    : the residuals from the quantile regression problem 
% beta : the solution to the quantile regression problem
% 
% The output from the function is
%
% xB   : A vector containing [beta abs(r(Ihc))]
% Ih   : The indexset s.t. beta=X(Ih)^(-1)*y(Ih), i.e. r(Ih)=0
% Ihc  : The complement of Ih, Ihc={1,2,...,n}\Ih
% P    : A vector with the elements equal sign(r(Ihc)), (with sign(0)=1)
%
% The above give the formulation
%
% [X(Ih)   0 ]*xB = y
% [X(Ihc) P*I]
%
% for an explanation of this see [1].
%
% References:
% [1] J. K. Mller (2006), Modeling of Unceartainty in Wind Energy
%        Forecast. Master Thesis, Informatics and Mathematical Modelling,
%        Technical University of Denmark. Available at
%        http://www.imm.dtu.dk/pubdb/p.php?4428. 
%
% [2] H. B. Nielsen (1999), Algorithms for Linear Optimization, an
%       Introduction. Course note for the DTU course "Optimization and Data
%       fitting 2". Available at http://www.imm.dtu.dk/courses/02611/ 


Index=1:n;   % Index is the indexset s.t. r=r(Index)

 % if the number of zero elements in r is equal rank(X), then the work is
 % essensially done, otherwise the if statement will find the index set Ih
 % s.t. X(Ih)*beta=y(Ih) and X(Ih)^(-1)*y(Ih)=beta, the important note being
 % that X(Ih) have an inverse. This is by using the LU transform of a
 % non-quadratic matrix.
 
if length(r(r==0))>length(beta)
    Lr0=sum(r==0);
    [rs,Irs]=sort(abs(r));
    Irs=Irs(1:Lr0);
    Xh=X(Irs,1:end);
    [L,U,P]=lu(Xh);
    In=1:Lr0;    
    for i=length(beta)+1:Lr0
        rI(i-length(beta))=In(P(i,1:end)==1);        
    end  
    r(Irs(rI))=10^-15;
end 

 % The indexsets, P and xB is defined 
Ih=Index(r==0); 
Ihc=Index(abs(r)>0);
P=sign(r(Ihc));
r(abs(r)<10^-15)=0;
xB=[beta;abs(r(Ihc))];
