function [CON,s,q,gain,md,alpha,h,IH,cq]=...
    rq_simplex_alg_final(Ih,Ihc,n,K,xB,Xny,IH,P,tau)
% Call:
%
%     [CON,s,q,gain,md,alpha,h,IH,cq]=...
%          rq_simplex_alg_final(Ih,Ihc,n,K,xB,Xny,IH,P,tau)
%
% rq_simplex_alg_final calculate the variables needed for a simplex
% algorithm for quantile regression. The function does not do the actual
% simplex step, this is done in the .m function rq_simplex_final.m. The
% algorithm is based on the recipe in [2], and a description of the 
% simplex algorithm for quantile regresion is given in [1]. 
%
% Input :
% Ih    : An index set that defines the guess on solution of the quantile 
%         regression problem. 
% Ihc   : The compliment of the index set Ih.
% n     : The number of observations in the design matrix.
% K     : The number of explanatory variables.
% xB    : The basic variables this is [beta, r(Ihc)*sign(r(Ihc))]'.
% Xny   : The design matrix which contain the the explanatory variables on
%         the training set 
% IH    : A matrix containing indexsets which have been visited by the
%         algorithm in rq_simplex_final, the indexsets in this can not be
%         returned.
% P     : The sign of the nonzero residuals, i.e. r(Ihc).
% tau   : The quantile to fit to.
%
% Output:
% CON   : The sondition number of the suggested X(Ih)'
% s     : The index to leave the Ih
% q     : The index to enter Ih 
% gain  : The improvement in the loss function from this step
% md    : The slope of the loss function from this direction.
% alpha : The amount that the solution can be moved in direction h.
% h     : The directional vector that the solution is changed in.
% IH    : An updated version of the input
% cq    : The sign of the new residual, if the solution is moved in
%         direction h.
%
% References:
% [1] J. K. Mller (2006), Modeling of Unceartainty in Wind Energy
%        Forecast. Master Thesis, Informatics and Mathematical Modelling,
%        Technical University of Denmark. Available at
%        http://www.imm.dtu.dk/pubdb/p.php?4428. 
%
% [2] H. B. Nielsen (1999), Algorithms for Linear Optimization, an
%       Introduction. Course note for the DTU course "Optimization and Data
%       fitting 2". Available at http://www.imm.dtu.dk/courses/02611/ 

invXh=Xny(Ih,1:end)^-1;
cB=(P<0)+P.*tau;
cC=[ones(K,1)*tau;ones(K,1)*(1-tau)];

IB2=-(P*ones(1,K).*Xny(Ihc,1:end))*invXh;

g=IB2'*cB;
d=cC-[g;-g];
d(abs(d)<10^-15)=0;
[md ,s]=sort(d);
s=s(md<0);
md=md(md<0);
c=ones(length(s),1);
c(s>K)=-1;
C=diag(c);
s(s>K)=s(s>K)-K;
h=[invXh(1:end,s);IB2(1:end,s)]*C;

alpha=0;
q=0;
xm=xB(K+1:end);
xm(xm<0)=0;
hm=h(K+1:end,1:end);
cq=0;
for k=1:length(s)  
    sigma=xm;
    sigma(hm(1:end,k)>10^-12)=...
        xm(hm(1:end,k)>10^-12)./hm(hm(1:end,k)>10^-12,k);
    sigma(hm(1:end,k)<=10^-12)=Inf;
    [alpha(k),q(k)]=min(sigma);  
    cq(k)=c(k);
end
gain=md'.*alpha;
[Mgain, IMgain]=sort(gain);
CON=Inf;
j=0;
if length(gain)==0
    gain=1;    
else
    while CON>10^6 & j<length(s)
        j=j+1;
        IhMid=Ih;
        IhMid(s(IMgain(j)))=Ihc(q(IMgain(j)));
        IhMid=sort(IhMid);
        if min(sum(abs(IH-IhMid'*ones(1,length(IH(1,1:end))))))==0
            CON=Inf;
        else
            CON=cond(Xny(IhMid,1:end));
        end   
    end
    s=s(IMgain(j));
    q=q(IMgain(j));
    cq=cq(IMgain(j));
    alpha=alpha(IMgain(j));
    IH=[IH,IhMid']; 
    h=h(1:end,IMgain(j));
    gain=gain(IMgain(j));
    md=md(IMgain(j));
end