function [N,BETA,GAIN,Ld,Rny,Mx,Re,CON1,T]...
    =rq_simplex_final(X,IX,Iy,Iex,r,beta,n,tau,bins,n_in_bin)

% Call:
%
% [N,BETA,GAIN,Ld,Rny,Mx,Re,CON1,T]...
%    =rq_simplex_final(X,IX,Iy,Iex,r,beta,n,tau,bins,n_in_bin)
%
% rq_simplex_final calculate the solution to an adaptive simplex 
% algorithm for a quantile regression problem. The function use 
% knowledge of the solution at time t to calculate the solution at 
% time t+1.
%
% The basic idea is that the solution to the quantile regression 
% problem can be written as 
%
%    y(t)=X(t)'*beta+r(t)
%
% where beta=X(h)^(-1)*y(h) for some indexset h, simplex algorithm 
% is now used to calculate the optimal h at time t+1 based on the solution
% at time t. So basically the function  uses the h(t) as a start guesss for
% the sipmplex algorithm to iterate to h(t+1). This function is described
% in the master thesis "Modelling of Uncertainty in Wind Energy Forecast"
% by Jan K. Moeller,  URL http://www.imm.dtu.dk/pubdb/p.php?4428.
%
%
% Iputs to the function
% X        : The design matrix for the linear quantile regression problem,
%            or rather it contains the design matrix, see the three next
%            input variables.
% IX       : An index set, it refer top the columns of X which is the
%            design matrix.
% Iy       : One index refer to the matrix X, the column Iy of X contains
%            the respons corresponding to the explanatory variables in the
%            design matrix
% Iex      : An index refering to a grouping variable, refer to a column of
%            X this may or may not be a part of the design matrix. This is
%            used in the updating algorithm. 
% r        : The residuals from a sulotion to the quantile regression
%            problem based on the first n (see below) elements of X. Such a
%            solution could be obtained by the "rq" method in "R". This is
%            only used to initialize the solution.
% beta     : The solution corresponding to the residuals in the vector r
%            above
% n        : The number of elements in r, i.e y(1:n)=X(1:n,IX)*beta+r(1:n) 
% tau      : The required probablity, the solution in r and beta should be
%            based on this probablity 
% bins     : A vector defining a partion of an interval which covers all 
%            elements in X(1:end,Iex). This is used for the updating
%            procedure of the design matrix. If bins=[-Inf,Inf] the the
%            updating procedure is on gliding window.
% n_in_bin : Number of elements in the bins defined above, this is one
%            number, so the number of elements will be the same in each of
%            the bin after some time.
%
% Remarks:
% This implementatins requires that the number of elements in each bin of
% the initial solution is less than or equal to n_in_bin.
%
% Output:
% N    : The number of simplex steps to get the solutions at time t+1 
%        given the solution at time t. This is a vector.
% BETA : A matrix with each column being the solution to quantile
%        regression problem. Corresponnding to the matrix X(n+1:end,1:end).
% GAIN : This is a parameter used only to analyze the method. It is the
%        gain in the loss function in each simplex step, if this is very
%        large at some points it can be taken as a sign of the problem
%        being ill possed.        
% Ld   : This is only used for analyzing the algorithm in its own right. It
%        gives the number of decent directions in each simplex step.
% Rny  : This is the rsidual of the one step ahead prediction of the
%        method. 
% Mx   : This is only used for analyzing the algorithm in its own right. It
%        is the minimum of the constraint solution to the simplex
%        formulation. This should be larger than or equal to zero. If this
%        becomes large in absolute value then it is an indication of the
%        algorithm having problems at this point
% Re   : The reliability on the training set for each point along the
%        solution, if everything is good, then this should follow Theorem
%        2.3 in the reference mentioned above. I.e. this is close to tau at
%        all points. 
% CON1 : The condition numeber of the matrices X(h(t))
% T    : The time used for each iteration
%
% References:
% [1] J. K. Mller (2006), Modeling of Unceartainty in Wind Energy
%        Forecast. Master Thesis, Informatics and Mathematical Modelling,
%        Technical University of Denmark. Available at
%        http://www.imm.dtu.dk/pubdb/p.php?4428. 
%
% [2] H. B. Nielsen (1999), Algorithms for Linear Optimization, an
%       Introduction. Course note for the DTU course "Optimization and Data
%       fitting 2". Available at http://www.imm.dtu.dk/courses/02611/ 



  % Initialize output vectors and  matrices 
GAIN=0;
Rny=0;
Ld=0;
mx=0;
T=0;
Mx=0;
N=zeros(1,2);
CON1=0;
CON2=[0,0];
BETA=beta';

 % Initialize internal variables for the function
 
K=length(beta); %K is the number of explanatory variables in X
 
Xny=X(1:n,1:end); %The design matrix at the starting point for the 
                  %algorithm, this will be the observation matrix in the
                  %simulation. So each solution will be based on Xny at the
                  %given time point.
                  
H=zeros(1,K);     %This is used to track solutions that the algorithm
                  %have visited

tolmx=10^-15;     %This is a tolerance to dertermine when an infeasible 
                  %point should be fixed and when it should be onsidered
                  %zero.
                  
j=0;              %Counter of the number of simplex steps in each 
                  %iteration 
                  
LX=length(X(1:end,Iy)); %The stop crterium of the simulation

i=2;                    %Internal counter used by the opdating procedure 
                        %decribed below 

k=0;                    %Timecounter for the algortihm, k will be the time
                        %from the beginning of the algorithm 
                        

                  
 %The function rq_initialiser_final initialize the parameters needed for 
 %the simplex algorithm based on the on the solution given as input to the
 %function
 
[xB,Ih,Ihc,P]=rq_initialiser_final(Xny(1:end,IX),r,beta,n);

  %The while loop makes the simulation through the data set, in each step
  %either i or n is updated. n is the total number of elements in the
  %design matrix, in the first iterations n will be updated later it will
  %be i.
  
while i+n<LX   
    k=k+1       %The time counter is updated
    t=cputime;   %Zero point for cpu-time 
 
 
    Re(k)=sum(P<0)/n;     %Reliability at time k
    mx=min(xB(K+1:end));  %The minimum of the basic solution, this should 
                          %be larger than zero
                          
    %Infeasible points is taken care of by rq_purify_final
    
    if j>0 & mx<-tolmx;
        [xB,P]=rq_purify_final(xB,Ih,Ihc,P,K,Xny(1:end,IX),Xny(1:end,Iy));
        mx=min(xB(K+1:end));
    end
    Mx(k)=mx; %The minimum of the basic solution is stored
    j=0;      %Simplex step counter is set to zero to start the new 
              %timestep 
              
    beta=xB(1:K); %The solution at time k is ekstracted and stored 
    
    BETA=[BETA;beta']; 
    
    %The design matrix and other variables needed for the simplex 
    %procedure is updated
    
    [Ih,Ihc,xB,Xny,Rny,P,n,i]=opdatering_final(X,Xny,IX,Iy,Iex,...
        Ih,Ihc,beta,Rny,K,n,xB,P,tau,i,bins,n_in_bin);
    
    %Infeasible points is set equal zero to avoid taken a simplex step in
    %this direction
    
    IH=Ih'; %The index set h is stored in each sipmlex step, this is done 
            %to prevent the solution from going back and furth between
            %equally good solutions
    
    %Numbers needed to perform the simplex step is calculated in
    %rq_simplex_alg_final

    [CON,s,q,gain,md,alpha,h,IH,cq]=...
        rq_simplex_alg_final(Ih,Ihc,n,K,xB,Xny(1:end,IX),IH,P,tau);
    CON1=[CON1;CON]; %The condition number of the matrix X(h(t+1)) is 
                     %stored
                     
    %The while loop continuie until the optimal solution is reached or one 
    %of the two stop criterias are violated. These are a maximal number of
    %simplex step, and the condition number of the next soltion
    
    while gain<=0 & md<0 & j<24 & CON<10^6
        GAIN=[GAIN;gain]; %The gain for the step in stored 
        j=j+1;            %The simplex counter is updated 
        z=xB-alpha*h;     %z is the new solution to the problem 
        
        %The index sets defining the solution is updated 
        IhM=Ih(s);
        IhcM=Ihc(q);
        Ih(s)=IhcM;
        Ihc(q)=IhM;
        P(q)=cq;    %The sign of the newest residual
        
        %The basic solution is updated.
        xB=z;    
        xB(q+K)=alpha;
        
        %The indexsets are ordered s.t. they are in increasing order 
        [Ih,IndexIh]=sort(Ih);
        [Ihc, IndexIhc]=sort(Ihc);
        P=P(IndexIhc);
        xBm=xB(K+1:end);
        xBm=xBm(IndexIhc);
        xB(K+1:end)=xBm;
        
        %rq_simplex_alg_final calculate the numbers needed to perform the
        %next simplex step
        [CON,s,q,gain,md,alpha,h,IH,cq]=...
            rq_simplex_alg_final(Ih,Ihc,n,K,xB,Xny(1:end,IX),IH,P,tau);
        
        CON1=[CON1;CON]; %The condition nember is stored 
    end
    
    N(k,1)=j;       %Number of simplex steps at time k
    N(k,2)=n;       %size of the design matrix at time k
    T(k)=cputime-t; %cpu time used to udate the solution
end